#include "tools.h"

double std(unsigned char *nums, int n){

  int i;
  double ms = 0;
  double mean = average(nums,n);

  for(i=0; i<n; i++){
    ms += pow(mean-nums[i],2);
  }
  return sqrt(ms/n);
}

double average(unsigned char *nums, int n){

  double sum = 0;
  int i = 0;
  for(i=0;i<n;i++){
    sum += nums[i];
  }
  return(sum/n);
}

void histogram_eq(unsigned char *in, int n){

  int cdf_in[256] = {0};
  int cdf_out[256] = {0};
  int i, j;
  unsigned char cdf_min = 255;

  // generate the input CDF (cumulative distribution function)
  for(i=0; i<n; i++){
    for(j = in[i]; j<256; j++){
      cdf_in[j] += 1;
    }
  }
  // find the minimum of CDF
  for(i=0; i<256; i++){
    if(cdf_in[i] < cdf_min){
      cdf_min = cdf_in[i];
    }
  }

  // find the output CDF
  for(i=0; i<256; i++){
    cdf_out[i] = round(255 * (cdf_in[i] - cdf_min)/(n - cdf_min));
  }

  // adjust the histogram
  for(i=0; i<n; i++){
    in[i] = cdf_out[in[i]];
  }
}

void stretch(unsigned char *in, int n, float p_low, float p_up){

  int i, sum;
  unsigned int hist[256];
  unsigned char lower_val = 0;
  unsigned char higher_val = 0;
  double p5 = n*p_low;
  double p95 = n*p_up;

  unsigned char hist_max = 0, hist_min = 0;

  for(i=0; i<256; i++){
    hist[i] = 0;
  }

  for(i=0; i<n; i++){
    hist[in[i]]++;
  }

  for(i=0; i<256; i++){ if(hist[i] > 0){ hist_max = i; } }
  for(i=255; i>-1; i--){ if(hist[i] > 0){ hist_min = i; } }

  // Check that we don't try to scale constant value image
  if(hist_max != hist_min){
    sum = 0;
    for(i=0; i<256; i++){
      sum += hist[i];
      if(sum < p5){
	lower_val++;
      }
      if(sum < p95){
	higher_val++;
      }
    }
    
    for(i=0; i<n; i++){
      p5 = 255*(in[i]-lower_val)/(higher_val-lower_val);
      if(p5 > 255){
	in[i] = 255;
      } else {
	if(p5 < 0) {
	  in[i] = 0;
	} else {
	  in[i] = p5;
	}
      }
    }
  }
}

int find_md_folder_num(char *basedir){

  char md_dir_name[FNAME_SIZE2] = {'\0'};
  char tmp[7] = {'\0'};

  int i = 0;

  for(i=0;i<NUM_MD_DIRS2;i++){

    strcpy(md_dir_name,basedir);
    sprintf(tmp,"MD%.4d",i);
    strcat(md_dir_name,tmp);

    if(!file_exists(md_dir_name)){
      return i;
    }
    memset(md_dir_name,'\0',FNAME_SIZE2);
  }

  return 0;

}

int file_exists(char *fname){

  struct stat st;

  stat(fname,&st);

  if(errno == ENOENT){
    return 0;
  }

  return 1;

}

unsigned char limit(double val){

  if(val < 0){
    return 0;
  }
  if(val > 255){
    return 255;
  }

  return (unsigned char)round(val);

}
